define([
	"dijit/registry",
	"aps/i18n",
	"aps/load",
	"aps/Message",
	"dojo/json"
], function(
	registry,
	_,
	load,
	Message,
	json
) {
	"use strict";

	var pageUnload = false;
	window.onbeforeunload = function() {
		pageUnload = true;
	};

	var getErrorMessage = function(err) {
		if (typeof err == 'string' || err instanceof String) {
			return err;
		} else {
			var errorMessage = "";
			var errData = {};
			try {
				errData = json.parse(err.response.text);
			} catch(e) {
			}
			if (errData.message) {
				var errDataMessage = errData.message.toLowerCase();
				if (errDataMessage.indexOf("<html>") > -1) {
					var bIndex = errDataMessage.indexOf("<title>");
					var eIndex = errDataMessage.indexOf("</title>");
					errorMessage = errData.error + ((bIndex > -1) && (eIndex > -1) ? " (" + errData.message.substring(bIndex + 7, eIndex) + ")" : "");
				} else {
					errorMessage = errData.message.replace("REST Application returns error code=500: ", "");
				}
				errorMessage = (!("status" in err) || err.status === null ? "" : "[" + err.status + "] ") + _(errorMessage);
			} else {
				errorMessage = "" + err;
			}
			return errorMessage;
		}
	};

	var displayError = function(value, clearCurrentMessages) {
		if (pageUnload) {
			return;
		}

		aps.apsc.cancelProcessing();
		aps.apsc.hideLoading();

		var errors = value instanceof Array ? value : [value];
		var messageCounts = {};

		for (var i = 0; i < errors.length; i++) {
			var errorMessage = getErrorMessage(errors[i]);
			messageCounts[errorMessage] = errorMessage in messageCounts ? messageCounts[errorMessage] + 1 : 1;
		}

		var createdMessages = [];
		for (var message in messageCounts) {
			if (messageCounts.hasOwnProperty(message)) {
				var msg = new Message({ description: message.replace(/\n/gm, '<br/>') + (messageCounts[message] > 1 ? (" (" + messageCounts[message] + " occurences)") : ""), type: "error" });
				createdMessages.push(msg);
			}
		}

		var showErrors = function(page, messages) {
			var messageList = page.get("messageList");
			if (clearCurrentMessages === undefined || clearCurrentMessages) {
				messageList.removeAll();
			}
			for (var i = 0; i < messages.length; i++) {
				messageList.addChild(messages[i]);
			}
		};

		var page = registry.byId("pageContainer");
		if (page == null) {
			load(["aps/PageContainer", { id: "pageContainer" }]).then(function() {
				page = registry.byId("pageContainer");
				showErrors(page, createdMessages);
			});
		} else {
			showErrors(page, createdMessages);
		}

		return value instanceof Array ? createdMessages : createdMessages[0];
	};

	return displayError;
});

