define([
	"aps/ResourceStore",
	"dojo/Deferred",
	"dojo/_base/array",
	"./utils.js",
	"aps/i18n"
], function(
	ResourceStore,
	Deferred,
	array,
	utils,
	_
){
	"use strict";

	var profileHelper = {

		profileAddressMapper: {
			address_line1:{
				prop_name: "streetAddress",
				label: _("Address Line 1")
			},
			address_line2:{
				prop_name: "extendedAddress",
				label: _("Address Line 2")
			},
			city:{
				prop_name: "locality",
				label: _("City")
			},
			region: {
				prop_name: "region",
				label: _("State")
			},
			state: {
				prop_name: "region",
				label: _("State")
			},
			postal_code:{
				prop_name: "postalCode",
				label: _("Zip Code")
			},
			country:{
				prop_name: "countryName",
				label: _("Country")
			}
		},

		_getMessageText: function(properties, isCombination){
			return _("__properties__ __combination__is invalid.<br/>Correct your entry at <b>Home &gt; Account &gt; Account Settings</b> and wait until the subscription provisioning is complete.", {
				properties: properties,
				combination: (isCombination ? _("combination ") : "")
			});
		},

		getErrorMessageRequest: function(propertyNames, tenantId){
			var self = this;
			var result = new Deferred();
			var accountStore = new ResourceStore({target: "/aps/2/resources/" + tenantId + "/core_account"});
			accountStore.query().then(function(accounts){
				if (accounts.length === 0){
					result.resolve({ text: _("Cannot obtain the account resource of the subscription.") });
					return result;
				}
				var account = accounts[0];
				var hasChanges = false;
				var props = array.map(propertyNames, function(propItem){
					if (self.profileAddressMapper.hasOwnProperty(propItem.name)){
						var property = self.profileAddressMapper[propItem.name];

						if (account.addressPostal.hasOwnProperty(property.prop_name)){
							var value = account.addressPostal[property.prop_name];

							if (propItem.value && propItem.value !== value){
								hasChanges = true;
							}
							return property.label + " \"" + value + "\"";
						}
						return property.label;
					}
					return "\"" + propItem + "\"";
				});
				var msgText = "";
				if (!hasChanges && props.length > 0) {
					msgText = self._getMessageText(utils.arrayToString(props), props.length > 1);
				}
				result.resolve({ text: msgText });
			}, result.reject);
			return result;
		}
	};

	return profileHelper;
});